/*
*       Quork C preprocessor, version 0.1
*       Anicka Bernathova, <anicka@anicka.net>, 05/2007
*       License: GNU GPLv2
*/


/* token.c: Tokens and operations on them */

/* token type */
enum ttype {
	IDENTIFIER,
	PARAM,
	LPAREN,
	RPAREN,
	HASH,
	HASHHASH,
	HEAD,
	NUMBER,
	STRING,
	FILENAME,
	CHAR,
	OPERATOR,
	WHITESPACE,
	WHITESPACENL,
	PLACEHOLDER
};

struct token {
	enum ttype type; /* type of the token */
	int gl; /* line number */	
	struct token* previous;
	struct token* next;
	char text[1]; /* complete text of the token */
};

struct token* tok_new(int len);

struct token* new_chain(void);
void free_chain(struct token* pt);
void push_token(struct token* token,struct token* tc);

void print_token_chain(struct token* tc);
void print_token_chain_plaintext(struct token* tc);
void print_reverse_chain(struct token* tc);
int token_chain_cmp(struct token* c1, struct token* c2);
struct token* clone_token(struct token* orig);
struct token* split_chain(struct token* oldh, struct token* chain);
struct token* skip_whitespace(struct token* chain);
void delete_token(struct token* pd);
void insert_token_after(struct token* pt, struct token* new);
struct token* split_token_chain_ft(struct token* from, struct token* to);
void insert_chain_after(struct token* pt, struct token* head);
struct token* clone_token_chain(struct token* orig);


/* lex.c */

extern int tok_inc_enable;

void __attribute__((noreturn)) err(char* msg);
void warn(char* msg);

void lex_init(char* filename);
void print_file_stack(void);
void switch_file(char* text, enum ttype t);

struct token* mk_token(void);
