/*
*       Quork C preprocessor, version 0.1
*       Anicka Bernathova, <anicka@anicka.net>, 05/2007
*       License: GNU GPLv2
*       
*       functions working with token lists 
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "quork.h"

/* token allocation */
struct token*
tok_new(int len)
{
	struct token* pt;
	if (!(pt=malloc(sizeof(struct token)+(len*sizeof(char)))))
		err("Low memory");
	return pt;
}

void
print_token_chain(struct token* tc)
{
	struct token* pc;

	for (pc=tc->next; pc!=tc; pc=pc->next)
		//printf("[%s]",pc->text);
		printf("[%s, %d]",pc->text,pc->type);
}

void
print_token_chain_plaintext(struct token* tc)
{
	struct token* pc;

	for (pc=tc->next; pc!=tc; pc=pc->next)
		printf("%s",pc->text);
}

void
print_reverse_chain(struct token* tc)
{
	struct token* pc;

	for (pc=tc->previous; pc!=tc; pc=pc->previous)
		//printf("[%s]",pc->text);
		printf("[%s, %d]",pc->text,pc->type);
}

/*
init new token chain - free with free_chain
initial type is HEAD but it might be changed to some other virtual token
*/
struct token*
new_chain(void)
{
	struct token* pt;

	if (!(pt=malloc(sizeof(struct token))))
		err("Low memory");
	pt->next=pt;
	pt->previous=pt;
	pt->type=HEAD;
	*pt->text='\0'; /* we might want to print even HEAD  */
	return pt;
	
}

void
push_token(struct token* token,struct token* tc)
{
	token->next=tc;
	token->previous=tc->previous;
	tc->previous->next=token;
	tc->previous=token;
}

/* free token chain */
void
free_chain(struct token* pt)
{
	struct token* temp, * t2;

	t2=pt->next;	
	while(t2->type!=HEAD){
		temp=t2->next;
		free(t2);
		t2=temp;	
	}
	free(t2);
}

struct token*
skip_whitespace(struct token* chain)
{
	while (chain->type==WHITESPACE
	  || chain->type==WHITESPACENL){
		chain=chain->next;	
	}

	return chain;
}

/*
Gets pointer into the token chain. Disconnects part from HEAD
to this pointer excluding it, add placeholder token instead the original chain,
creates token chain again and returns it

Fixes the resting tokens to be the valid token chain.
*/
struct token*
split_chain(struct token* oldh, struct token* chain)
{
	struct token* newh, * plh, * pt, * pt2;

	newh=new_chain();

	plh=new_chain(); /* inits new token with type HEAD and without space for text*/
	plh->type=PLACEHOLDER;	

	pt=chain->previous;
	pt2=oldh->previous;

	pt->next=plh;
	plh->previous=pt;

	plh->next=oldh;
	oldh->previous=plh;
	
	pt2->next=newh;
	chain->previous=newh;
	newh->previous=pt2;
	newh->next=chain;

	return newh;
}

struct token*
clone_token(struct token* orig)
{

	struct token* new;

	if (!(new=malloc(sizeof(struct token)+strlen(orig->text)+1)))
		err("Low memory");

	*new = *orig;
	strcpy(new->text, orig->text);	

	return new;

}

struct token*
clone_token_chain(struct token* orig)
{
	struct token* pt, * ret, * new;

	pt=orig->next;
	ret=new_chain();
	while (pt->type!=HEAD){
		new=clone_token(pt);
		insert_token_after(ret->previous, new);
		pt=pt->next;
	}

	return ret;
}
/*
compares two macro definitions
whitespace should be ignored
*/
int
token_chain_cmp(struct token* c1, struct token* c2)
{
	struct token* pt, * pt2;

	pt2=skip_whitespace(c2->next);
	for (pt=c1->next; pt->type==HEAD; pt=skip_whitespace(pt->next)){
		if (pt->type==pt2->type &&
		  !strcmp(pt->text,pt2->text)){
			if (!(pt->type==PARAM && pt->gl==pt2->gl))
				return 0; /* check that params have the same spelling */
		}
		else
			return 0;

		pt2=skip_whitespace(pt2->next);
	}

	return 1;
}


void
delete_token(struct token* pd)
{
	pd->previous->next=pd->next;
	pd->next->previous=pd->previous;	

	free(pd);		
}

void
insert_token_after(struct token* pt, struct token* new)
{
	new->next=pt->next;
	new->previous=pt;
	pt->next->previous=new;
	pt->next=new;
}

void
insert_chain_after(struct token* pt, struct token* head)
{
	head->previous->next=pt->next;
	head->next->previous=pt;
	pt->next->previous=head->previous;
	pt->next=head->next;

	free(head);
}

struct token*
split_token_chain_ft(struct token* from, struct token* until)
{
	struct token* ret;
	ret=new_chain();	

	if (from==until)
		return ret;

	ret->next=from;
	ret->previous=until->previous;
	from->previous->next=until;
	until->previous=from->previous;
	from->previous=ret;
	ret->previous->next=ret;	
	
	return ret;
}
